/*
	Radino-WiFi-Firmwareupdate Example for In-Circuit radino WiFi modules
        for more information: www.in-circuit.de or www.radino.cc

	Copyright (c) 2014 In-Circuit GmbH

	Permission is hereby granted, free of charge, to any person obtaining a copy of this software
	and associated documentation files (the "Software"), to deal in the Software without restriction,
	including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
	and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
	LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
	IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
	WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
/*  
  This example demonstrates how to build a WiFi-UART-Bridge with
  radino WiFi and setup the ESP8266 for a Firmwareupdate.
    
  This example enables you to directly communicate with the bootloader of the
  ESP8266 WiFi-chip which is integrated on the radino WiFi. It can be used to
  upload new firmware to the ESP8266 WiFi-module.
  
  The SC16IS750 enables the use of baudrates greater than 57600 when
  communicating with ESP8266. (The Arduino runs at 8 Mhz, therfore limited
  to baudrates less than 57600)
  
- radino WiFi setup:
  
  Arduino <---SPI---> SC16IS750 <---UART---> ESP8266
  
- Internal connections between Arduino(ATmega32U4),SC16IS750 and ESP8266:
  
  Arduino          ESP8266        SC16IS750
  
  A4               CH_PD          ---
  D4               RST            ---
  D9               GPIO0          ---
  ---              GPIO2          GPIO0
  
- Pin setup for ESP8266:

  Pin            normal operation        Firmware-Update-Mode
  
  CH_PD          HIGH                    HIGH
  GPIO0          HIGH                    LOW
  GPIO2          HIGH                    HIGH
  RST            HIGH                    HIGH
  
 */
 
// Define Serial port that should be used for communication
// Valid values:
// USB-UART:       Serial
// Hardware-UART:  Serial1
#define progSerial Serial1

// Use SPI-library for communication with SC16IS750
#include <SPI.h>

// Include definitions and functions for SC16IS750
// SPI-to-UART-chip
#include <SPI_UART.h>

// In-Circuit ESP-library
#include <IC_ESP.h>

IC_ESP esp = IC_ESP();

void setup()
{
  progSerial.begin(57600);  // Start serial port at baudrate 57600
  delay(100);
  
  // Init ESP8266EX
  esp.init();
  esp.setMode(ESP_FIMWAREUPDATE_MODE); // Put ESP8266 into FirmwareUpdateMode
}

/*
 * "Connect" external UART to SPI-UART
 * This enables direct communication with ESP8266 WiFi-Chip
*/
void loop()
{ 
  char data = '0';
  unsigned char cnt = 0;
  
  // If data available from ESP8266EX, pass through to external UART
  if(cnt = esp.available())
  {
    for(cnt; cnt>0; cnt--)
    {
      data = esp.read();
      progSerial.write(data); 
    }
  }
  
  // If data available from external UART, pass through to ESP8266EX
  if(progSerial.available())
  {
    data = progSerial.read();
    esp.write(data); 
  }
}

